/*
 * Written by Sean Kelly
 * Placed into Public Domain
 */

module std.error;

private
{
    import std.c.stdio;
    import std.c.stdlib;
}

private
{
    class InternalError( alias msg )
    {
        char[]  file;
        uint    line;

        this( char[] file, uint line )
        {
            this.file = file;
            this.line = line;
        }

        char[] toString()
        {
            return msg;
        }
    }

    static char[] arrayBoundsErrorMsg     = "ArrayBoundsError";
    static char[] assertErrorMsg          = "AssertError";
    static char[] outOfMemoryErrorMsg     = "OutOfMemoryError";
    static char[] switchErrorMsg          = "SwitchError";

    alias void function( char[] file, uint line ) assertHandlerType;
    alias void function() terminateHandlerType;

    static assertHandlerType assertHandler = null;
}

alias InternalError!( arrayBoundsErrorMsg ) ArrayBoundsError;
alias InternalError!( assertErrorMsg )      AssertError;
alias InternalError!( outOfMemoryErrorMsg ) OutOfMemoryError;
alias InternalError!( switchErrorMsg )      SwitchError;

void setAssertHandler( assertHandlerType h )
{
    assertHandler = h;
}

extern (C) void onAssert( char[] file, uint line )
{
    if( assertHandler )
        assertHandler( file, line );
    else
        throw new AssertError( file, line );
}

// NOTE: currently, the rest of these functions are for internal use and
//       meant to eliminate object dependencies between the runtime and
//       the standard lib.

extern (C) void onOutOfMemory()
{
    throw cast(OutOfMemoryError) cast(void*) OutOfMemoryError.classinfo.init;
}

extern (C) void onArrayBoundsError( char[] file, size_t line )
{
    throw new ArrayBoundsError( file, line );
}

extern (C) void onSwitchError( char[] file, size_t line )
{
    throw new SwitchError( file, line );
}

extern (C) void onInvalidUtfError( char[] msg, size_t idx )
{
    throw new InvalidUtfError( msg, idx );
}

class InvalidUtfError : Exception
{
    size_t idx;

    this( char[] msg, size_t idx )
    {
        super( msg );
        this.idx = idx;
    }
}