# Makefile to build D runtime library ares.lib for Win32
# Designed to work with \dm\bin\make.exe
# Targets:
#	make
#		Same as make unittest
#	make ares.lib
#		Build ares.lib
#	make clean
#		Delete unneeded files created by build process
#	make unittest
#		Build ares.lib, build and run unit tests
# Notes:
#	This relies on LIB.EXE 8.00 or later, and MAKE.EXE 5.01 or later.

CP=copy /y

CFLAGS=-mn -6 -r -Igc
#CFLAGS=-g -mn -6 -r -Igc

DFLAGS=-release -O -inline
#DFLAGS=-g -release

TFLAGS=-O -inline
#TFLAGS=-g

CC=\bin\dm\bin\sc
DMD=\bin\dmd\bin\dmd

.c.obj:
	$(CC) -c $(CFLAGS) $*

.cpp.obj:
	$(CC) -c $(CFLAGS) $*

.d.obj:
	$(DMD) -c $(DFLAGS) $*

.asm.obj:
	$(CC) -c $*

targets : ares

ares : ares.lib

######################################################

OBJ_STD= atomic.obj error.obj intrinsic.obj thread.obj

OBJ_STD_C= complex.obj ctype.obj errno.obj fenv.obj inttypes.obj \
    math.obj signal.obj stdarg.obj stdbool.obj stddef.obj stdint.obj  \
    stdlib.obj string.obj time.obj wctype.obj
### linking in stdio breaks i/o
# stdio.obj

OBJ_STD_C_POSIX= inttypes.obj unistd.obj

OBJ_STD_C_POSIX_SYS= types.obj

OBJ_SYS_C_WIN= com.obj process.obj windows.obj

OBJ_SYS_C_LINUX=linux.obj linuxextern.obj

######################################################

ares.lib : $(OBJ_STD) $(OBJ_STD_C) $(OBJ_SYS_C_WIN)
    del ares.lib
	lib -c -n ares.lib $(OBJ_STD) $(OBJ_STD_C) $(OBJ_SYS_C_WIN)

######################################################

### std

atomic.obj : std\atomic.d
    $(DMD) -c $(TFLAGS) std\atomic.d

error.obj : std\error.d
    $(DMD) -c $(DFLAGS) std\error.d

intrinsic.obj : std\intrinsic.d
    $(DMD) -c $(DFLAGS) std\intrinsic.d

thread.obj : std\thread.d
	$(DMD) -c $(DFLAGS) std\thread.d

### std\c

complex.obj : std\c\complex.d
	$(DMD) -c $(DFLAGS) std\c\complex.d

ctype.obj : std\c\ctype.d
	$(DMD) -c $(DFLAGS) std\c\ctype.d

errno.obj : std\c\errno.d
	$(DMD) -c $(DFLAGS) std\c\errno.d

fenv.obj : std\c\fenv.d
	$(DMD) -c $(DFLAGS) std\c\fenv.d

inttypes.obj : std\c\inttypes.d
	$(DMD) -c $(DFLAGS) std\c\inttypes.d

math.obj : std\c\math.d
	$(DMD) -c $(DFLAGS) std\c\math.d

signal.obj : std\c\signal.d
	$(DMD) -c $(DFLAGS) std\c\signal.d

stdarg.obj : std\c\stdarg.d
	$(DMD) -c $(DFLAGS) std\c\stdarg.d

stdbool.obj : std\c\stdbool.d
	$(DMD) -c $(DFLAGS) std\c\stdbool.d

stddef.obj : std\c\stddef.d
	$(DMD) -c $(DFLAGS) std\c\stddef.d

stdint.obj : std\c\stdint.d
	$(DMD) -c $(DFLAGS) std\c\stdint.d

stdio.obj : std\c\stdio.d
	$(DMD) -c $(DFLAGS) std\c\stdio.d

stdlib.obj : std\c\stdlib.d
	$(DMD) -c $(DFLAGS) std\c\stdlib.d

string.obj : std\c\string.d
	$(DMD) -c $(DFLAGS) std\c\string.d

time.obj : std\c\time.d
	$(DMD) -c $(DFLAGS) std\c\time.d

wctype.obj : std\c\wctype.d
	$(DMD) -c $(DFLAGS) std\c\wctype.d

### sys\c\windows

com.obj : sys\c\windows\com.d
	$(DMD) -c $(DFLAGS) sys\c\windows\com.d

process.obj : sys\c\windows\process.d
	$(DMD) -c $(DFLAGS) sys\c\windows\process.d

windows.obj : sys\c\windows\windows.d
	$(DMD) -c $(DFLAGS) sys\c\windows\windows.d

######################################################

clean :
	del $(OBJ_STD) $(OBJ_STD_C) $(OBJ_SYS_C_WIN) $(OBJ_STD_C_UNIX) \
	$(OBJ_STD_C_UNIX_SYS) $(OBJ_STD_C_LINUX)
	del ares.lib

install :
	$(CP) ares.lib ..\..\lib\.
