/*
 *  Copyright (C) 2004-2005 by Digital Mars, www.digitalmars.com
 *  Written by Walter Bright
 *
 *  This software is provided 'as-is', without any express or implied
 *  warranty. In no event will the authors be held liable for any damages
 *  arising from the use of this software.
 *
 *  Permission is granted to anyone to use this software for any purpose,
 *  including commercial applications, and to alter it and redistribute it
 *  freely, in both source and binary form, subject to the following
 *  restrictions:
 *
 *  o  The origin of this software must not be misrepresented; you must not
 *     claim that you wrote the original software. If you use this software
 *     in a product, an acknowledgment in the product documentation would be
 *     appreciated but is not required.
 *  o  Altered source versions must be plainly marked as such, and must not
 *     be misrepresented as being the original software.
 *  o  This notice may not be removed or altered from any source
 *     distribution.
 */

module object;

private
{
    import std.c.string;
}

public import std.c.stddef;
public import std.c.stdbool;

class Object
{
    char[] toString()
    {
	    return this.classinfo.name;
    }

    uint toHash()
    {
	    // BUG: this prevents a compacting GC from working, needs to be fixed
	    return cast(uint)cast(void *)this;
    }

    int opCmp(Object o)
    {
	    // BUG: this prevents a compacting GC from working, needs to be fixed
	    return cast(int)cast(void *)this - cast(int)cast(void *)o;
    }

    int opEquals(Object o)
    {
	    return this is o;
    }
}

struct Interface
{
    ClassInfo   classinfo;
    void*[]     vtbl;
    int         offset;		// offset to Interface 'this' from Object 'this'
}

class ClassInfo : Object
{
    byte[]      init;		// class static initializer
    char[]      name;		// class name
    void*[]     vtbl;		// virtual function pointer table
    Interface[] interfaces;
    ClassInfo   base;
    void*       destructor;
    void(*classInvariant)(Object);
    uint        flags;
    // 1: // IUnknown
    void*       deallocator;
}

class TypeInfo
{
    uint toHash()
    {
        uint hash;

	    foreach (char c; this.classinfo.name)
	        hash = hash * 9 + c;
	    return hash;
    }

    int opCmp(Object o)
    {
        uint len = this.classinfo.name.length;
        int result;

        if (o.classinfo.name.length < len)
	        len = o.classinfo.name.length;
        result = memcmp(this.classinfo.name, o.classinfo.name, len);
        if (result == 0)
	        result = cast(int)this.classinfo.name.length - cast(int)o.classinfo.name.length;
        return result;
    }

    int opEquals(Object o)
    {
	    /* TypeInfo instances are singletons, but duplicates can exist
	     * across DLL's. Therefore, comparing for a name match is
	     * sufficient.
	     */
	    return this is o || this.classinfo.name == o.classinfo.name;
    }

    uint getHash(void *p) { return cast(uint)p; }
    int equals(void *p1, void *p2) { return p1 == p2; }
    int compare(void *p1, void *p2) { return 0; }
    int tsize() { return 0; }

    void swap(void *p1, void *p2)
    {
	    int i;
	    int n = tsize();
	    for (i = 0; i < n; i++)
	    {
	        byte t = (cast(byte *)p1)[i];
	        (cast(byte *)p1)[i] = (cast(byte *)p2)[i];
	        (cast(byte *)p2)[i] = t;
	    }
    }
}

class Exception : Object
{
    char[]      msg;
    char[]      file;
    size_t      line;
    Exception   next;

    this(char[] msg, Exception next = null)
    {
        this.msg = msg;
        this.next = next;
    }

    this(char[] msg, char[] file, size_t line, Exception next = null)
    {
        this(msg, next);
        this.file = file;
        this.line = line;
    }

    char[] toString() { return msg; }
}